import shlex
import json
import subprocess
import json
import html
import psycopg2 as pg
from sqlalchemy import create_engine
from psycopg2.extras import execute_values
from multiprocessing import Process, Queue, Array, current_process
from underthesea import text_normalize
from bs4 import BeautifulSoup
import requests
import unidecode
import os
import numpy as np
import pandas as pd
import re
import urllib.request as urllib
import warnings
warnings.filterwarnings('ignore')
os.environ['http_proxy'] = "http://proxy.hcm.fpt.vn:80"
os.environ['https_proxy'] = "http://proxy.hcm.fpt.vn:80"
import yaml
from .config import infra_analytics_config

def get_config(config_file):
    with open(config_file, "r") as cf:
        config = yaml.load(cf, Loader=yaml.SafeLoader)
    return config
    
def infra_crawlershopftel(config: dict = infra_analytics_config):
    """
    Crawler thông tin văn phòng giao dịch của Ftel:
        + crawler thông tin văn phòng giao dịch Ftel từ: https://ftel.vn/getStoreBrach
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
        + Drop dữ liệu duplicate 
        + Upsert thông tin văn phòng giao dịch Ftel vào postgresql 177 - dwh_noc - public.tbl_shopfpttelecom_info
    """
    if not config: 
        config_file = "./config.yaml"
        config = get_config(config_file)
    cURL = r"""curl 'https://ftel.vn/getStoreBrach' \
    -H 'authority: ftel.vn' \
    -H 'accept: */*' \
    -H 'accept-language: vi-VN,vi;q=0.9,fr-FR;q=0.8,fr;q=0.7,en-US;q=0.6,en;q=0.5' \
    -H 'cookie: _gcl_au=1.1.1026824109.1701587762; _gid=GA1.2.802365421.1701587762; _gat_gtag_UA_245503180_1=1; _gat=1; _ga=GA1.1.1105386993.1701587762; XSRF-TOKEN=eyJpdiI6IkkvRTlWeGJDdU9PR3M5QnkwUlNBN1E9PSIsInZhbHVlIjoiS3hPSDlSaGt0V2lIYTk4TXFua0NqR0FKeUQzL0lEUWw2djVsNzc4T1hSNDI5OW1yR0c3WVdJRFM0WXdVbEV3Zy94YUJ5aFVwRDNkeVJFbEF1ZWt2UmZmcm1hTDRNUVlvcFhtTU1NMzFsZGl4SVRGMlR4ZWgveWlUQXZnZ212SjIiLCJtYWMiOiI3NWUyMzI5N2E3NGY5MzQzZTQyNTk3OTNmMjFjYjljZTJkZGU4NTBkMDFlZjg1NzU0ZWUwZjdkOTNiMGJmODBjIn0%3D; fleetcart_session=eyJpdiI6Imh2Q1d6anoyZnJHUFpLYjlkTXhjZHc9PSIsInZhbHVlIjoibEo4c0dxZW1EM2lHNEowd0JiWGJoOVFnRkhOWk1GWHRNMkdtckh5Tm10Z1VKd1Mxc0pqVUd2WTlyeDJpVjV1aTdMRVFUM2VaZDU2Z0JZRURRMnROcXpqMHlKZCs4T28yd0h0cUhyNmFXYkFvTXR6NVB6WEhEeGxXREkyaUlQUkUiLCJtYWMiOiI1Yjk1ZTdjZGRlYWRjZjIyYTMzOGVmYWQ1ZjliNDJkNWYxYWQ5ZmM1MDM5NDg5ZTFhMTU2Y2Q5ZDk4MmFlYzcyIn0%3D; _ga_H27VLGD6XJ=GS1.1.1701587761.1.1.1701587809.0.0.0; _ga_34B604LFFQ=GS1.1.1701587764.1.1.1701587809.15.0.0' \
    -H 'referer: https://ftel.vn/ho-tro/lien-he-24-7/diem-giao-dich' \
    -H 'sec-ch-ua: "Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24"' \
    -H 'sec-ch-ua-mobile: ?0' \
    -H 'sec-ch-ua-platform: "macOS"' \
    -H 'sec-fetch-dest: empty' \
    -H 'sec-fetch-mode: cors' \
    -H 'sec-fetch-site: same-origin' \
    -H 'user-agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36' \
    -H 'x-requested-with: XMLHttpRequest' \
    --compressed"""
    lCmd = shlex.split(cURL) # Splits cURL into an array
    p = subprocess.Popen(lCmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate() # Get the output and the err message
    json_data = json.loads(out.decode("utf-8"))
    lst_data_single = pd.DataFrame(json_data)
    df_vpgd = lst_data_single[['address']]
    df_vpgd['province'] = df_vpgd['address'].str.split(',').str[-1]
    df_vpgd['district'] = df_vpgd['address'].str.split(',').str[-2]
    df_vpgd['ward'] = df_vpgd['address'].str.split(',').str[-3]
    df_vpgd['street'] = df_vpgd['address'].str.split(',').str[:-3]
    df_vpgd['street'] = [','.join(map(str, l)) for l in df_vpgd['street']]
    df_vpgd = df_vpgd[['address','province','district','ward']]
    df_vpgd.drop_duplicates(keep='first',inplace=True)
    lst_data = df_vpgd.values.tolist()
    conn =  pg.connect("postgresql://%s:%s@%s:%s/%s"% (config['dbs']['dwh_177_public']['user']
                                                      ,config['dbs']['dwh_177_public']['password']
                                                     ,config['dbs']['dwh_177_public']['host']
                                                    ,config['dbs']['dwh_177_public']['port']
                                                    ,config['dbs']['dwh_177_public']['dbname']))
    cur_1 = conn.cursor()
    for i in range(len(lst_data)):
        tupl = (lst_data[i])
        cur_1.execute(
        " INSERT INTO tbl_shopfpttelecom_info(address,province,district, ward) VALUES( %s, %s, %s, %s)"
        " ON CONFLICT (address)"
        " DO UPDATE SET province = EXCLUDED.province, district = EXCLUDED.district, ward = EXCLUDED.ward;", (tuple(tupl))
        )
    try:
        conn.commit()
        conn.close()
        print("Successfully!!!!")
    except:
        print("Don't save DB")


import os
import spark_sdk as ss
ss.__version__
import os
ss.PySpark(yarn=False, num_executors=60, driver_memory='60g', executor_memory='24g',
            add_on_config1=('spark.port.maxRetries', '1000'))
spark = ss.PySpark().spark

from pyspark.sql.window import Window
from pyspark.sql.functions import row_number
import pyspark.pandas as ps
from pyspark.sql.functions import to_date,col
spark.sql("SET spark.sql.sources.partitionOverwriteMode = dynamic")
from datetime import timedelta
from datetime import datetime
from dateutil.relativedelta import relativedelta
import pandas as pd
import numpy as np
import glob, re

import sh
import requests
import pandas as pd
import psycopg2 as pg
from sqlalchemy import create_engine

import yaml
from .config import infra_analytics_config

def get_config(config_file):
    with open(config_file, "r") as cf:
        config = yaml.load(cf, Loader=yaml.SafeLoader)
    return config
    
def extract_date(file_path):
    # Split tên file để lấy phần ngày tháng
    file_name = file_path.split('/')[-1]  # Lấy phần cuối cùng sau dấu '/'
    date_str = file_name.split('_')[-1]   # Lấy phần sau dấu '_' chứa ngày tháng
    date_str = date_str.replace('.xlsx','')
    # Chuyển đổi từ string sang datetime
    return datetime.strptime(date_str, '%Y%m%d')
def process_hodannam(year,config):
    """
    Tổ chức lưu trữ thông tin hộ dân hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/ho_dan 
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/ho_dan.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['hodan_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df.columns = ['Mã tỉnh', 'Tỉnh', 'Mã quận', 'Quận', 'Mã phường', 'Phường', 'Tổng hộ',
               'Thành thị', 'Nông thôn', 'Năm']
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['hodan_path_output'])\
        .save()

def process_dansonam(year,config):
    """
    Tổ chức lưu trữ thông tin dân số hàng năm:
        + input:/data/fpt/ftel/infra/dwh/import_data_phattrienhatang/dan_so
        + output:/data/fpt/ftel/infra/dwh/phat_trien_ha_tang/dan_so.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['danso_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df.columns = ['Mã tỉnh', 'Tỉnh', 'Mã quận', 'Quận', 'Mã phường', 'Phường', 'Tổng số dân',
           'Dân số Nông thôn', 'Dân số Thành thị', 'Năm']
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['danso_path_output'])\
        .save()
def process_thanhphandansonam(year,config):
    """
    Tổ chức lưu trữ thông tin dân số hàng năm:
        + input:/data/fpt/ftel/infra/dwh/import_data_phattrienhatang/thanh_phan_dan_so
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/thanh_phan_dan_so.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['thanhphandanso_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df.columns = ['Mã tỉnh', 'Tỉnh', 'Mã quận', 'Quận', 'Mã phường', 'Phường',
           'Dưới tiểu học', 'Tiểu học', 'Trung học', 'Cao đẳng', 'Đại học',
           'Thạc sỹ', 'Tiến sỹ', 'Năm']
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['thanhphandanso_path_output'])\
        .save()

def process_nganhangnam(year,config):
    """
    Tổ chức lưu trữ thông tin ngân hàng hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/ngan_hang
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/ngan_hang.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['nganhang_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['nganhang_path_output'])\
        .save()
        
def process_chitieunam(year,config):
    """
    Tổ chức lưu trữ thông tin bình quân chi tiêu hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/chi_tieu
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/chi_tieu.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['chitieu_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['chitieu_path_output'])\
        .save()

def process_benhviennam(year,config):
    """
    Tổ chức lưu trữ thông tin bệnh viện hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/benh_vien
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/benh_vien.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['benhvien_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['benhvien_path_output'])\
        .save()

def process_trung_tam_y_tenam(year,config):
    """
    Tổ chức lưu trữ thông tin trung tâm y tế hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/trung_tam_y_te
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/trung_tam_y_te.parquet 
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['yte_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['yte_path_output'])\
        .save()
def process_truong_tieu_hocnam(year,config):
    """
    Tổ chức lưu trữ thông tin trường tiểu học hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/truong_tieu_hoc
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/truong_tieu_hoc.parquet
    """

    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['tieuhoc_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['tieuhoc_path_output'])\
        .save()
def process_truong_trung_hoc_co_sonam(year,config):
    """
    Tổ chức lưu trữ thông tin trường trung học cơ sở hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/truong_trung_hoc_co_so
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/truong_trung_hoc_co_so.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['trunghoccoso_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['trunghoccoso_path_output'])\
        .save()
def process_truong_trung_hoc_pho_thongnam(year,config):
    """
    Tổ chức lưu trữ thông tin trường trung học phổ thông hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/truong_trung_hoc_pho_thong
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/truong_trung_hoc_pho_thong.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['trunghocphothong_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['trunghocphothong_path_output'])\
        .save()

def process_truong_dai_hocnam(year,config):
    """
    Tổ chức lưu trữ thông tin trường đại học hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/truong_dai_hoc
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/truong_dai_hoc.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['truongdaihoc_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['truongdaihoc_path_output'])\
        .save()
def process_truong_cao_dangnam(year,config):
    """
    Tổ chức lưu trữ thông tin trường cao đẳng hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/truong_cao_dang
        + output:/data/fpt/ftel/infra/dwh/phat_trien_ha_tang/truong_cao_dang.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['truongcaodang_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['truongcaodang_path_output'])\
        .save()
def process_chonam(year,config):
    """
    Tổ chức lưu trữ thông tin chợ hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/cho
        + output:/data/fpt/ftel/infra/dwh/phat_trien_ha_tang/cho.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['cho_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['cho_path_output'])\
        .save()
def process_sieu_thinam(year,config):
    """
    Tổ chức lưu trữ thông tin siêu thị hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/sieu_thi
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/sieu_thi.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['sieuthi_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['sieuthi_path_output'])\
        .save()

def process_trung_tam_thuong_mainam(year,config):
    """
    Tổ chức lưu trữ thông tin trung tâm thương mại hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/trung_tam_thuong_mai
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/trung_tam_thuong_mai.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['trungtamthuongmai_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['trungtamthuongmai_path_output'])\
        .save()
def process_doanh_nghiepnam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghiep_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghiep_path_output'])\
        .save()
def process_doanh_nghiep_vua_va_nhonam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp vừa và nhỏ hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep_vua_va_nho
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep_vua_va_nho.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghiepvuanho_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghiepvuanho_path_output'])\
        .save()

def process_doanh_nghiep_tu_nhannam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp tư nhân hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep_tu_nhan
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep_tu_nhan.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghieptunhan_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghieptunhan_path_output'])\
        .save()
def process_khach_sannam(year,config):
    """
    Tổ chức lưu trữ thông tin khách sạn hàng năm:
        + input:/data/fpt/ftel/infra/dwh/import_data_phattrienhatang/khach_san
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/khach_san.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['khachsan_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['khachsan_path_output'])\
        .save()
def process_khach_san_tu_nhannam(year,config):
    """
    Tổ chức lưu trữ thông tin khách sạn tư nhân hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/khach_san_tu_nhan
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/khach_san_tu_nhan.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['khachsantunhan_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['khachsantunhan_path_output'])\
        .save()
def process_doanh_nghiep_co_von_nuoc_ngoainam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp có vốn nước ngoài hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep_co_von_nuoc_ngoai
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep_co_von_nuoc_ngoai.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghiepnuocngoai_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghiepnuocngoai_path_output'])\
        .save()
def process_doanh_nghiep_co_hoat_dong_xuat_nhap_khaunam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp có hoạt động xuất nhập khẩu hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep_co_hoat_dong_xuat_nhap_khau
        + output:/data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep_co_hoat_dong_xuat_nhap_khau.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghiepxuatnhapkhau_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghiepxuatnhapkhau_path_output'])\
        .save()
def process_doanh_nghiep_cong_nghe_thong_tinnam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp công nghệ thông tin hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep_cong_nghe_thong_tin
        + output:/data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep_cong_nghe_thong_tin.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghiepcntt_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghiepcntt_path_output'])\
        .save()
def process_doanh_nghiep_co_trang_thong_tin_dien_tunam(year,config):
    """
    Tổ chức lưu trữ thông tin doanh nghiệp có trang thông tin điện tử hàng năm:
        + input: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/doanh_nghiep_co_trang_thong_tin_dien_tu
        + output: /data/fpt/ftel/infra/dwh/phat_trien_ha_tang/doanh_nghiep_co_trang_thong_tin_dien_tu.parquet
    """
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['doanhnghiepttdt_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file)
    df['Năm'] = year
    df_ = spark.createDataFrame(df.to_pandas())
    df_ = df_.withColumn('d', to_date(col('Năm')))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('d')\
        .option("path", config['data_import']['doanhnghiepttdt_path_output'])\
        .save()
# Hàm để lấy dữ liệu từ một trang cụ thể
def fetch_data(page, limit, config:dict = infra_analytics_config):
    if not config:
        config_file = "./config.yaml"
        config_api = get_config(config_file)
    url = config_api['data_import']['api_getstarpop']+ f'?page={page}&limit={limit}'
    headers = {
    'content-type': 'application/json',
    }
    proxies = {
    "http_proxy": "http://proxy.hcm.fpt.vn:80",
    "https_proxy": "http://proxy.hcm.fpt.vn:80",
    "no_proxy": "icdp.fpt.net"
    }
    response = requests.get(url,
                  headers=headers, 
                  proxies=proxies)
    if response.status_code == 200:
        return response.json()
    else:
        return []

# Hàm để lấy toàn bộ dữ liệu từ API
def fetch_all_data(base_url, limit=10000):
    all_data = []
    page = 1
    while True:
        print(page)
        data = fetch_data(page, limit)
        if (len(data['data']['data'])==0):  # Nếu không còn dữ liệu mới, dừng lại
            break
        all_data.extend(data['data']['data'])
        page += 1
    return all_data
def process_get_star_pop(month,config):
    """
    Tổ chức lưu trữ thông tin đánh giá chất lượng POP theo thời gian:
    + Get data từ api https://icdp.fpt.net/flask/cads/get_star_pop
    + Lưu dữ liệu xuống postgresql 177 - dwh_noc - public.tbl_quality_pop
    """
    # Sử dụng hàm để lấy toàn bộ dữ liệu
    base_url = config['data_import']['api_getstarpop']
    all_data = fetch_all_data(base_url)
    lst_data_single = pd.DataFrame(all_data)
    lst_data_single.columns=['month','avg_operation_pop','avg_quality_pop','pop','province']
    lst_data_single['month'] = pd.to_datetime(lst_data_single['month'])
    tablename_pop = config['data_import']['tablename_qualitypop']
    conn = pg.connect("postgresql://%s:%s@%s:%s/%s"% (config['dbs']['dwh_177_public']['user']
                                                      ,config['dbs']['dwh_177_public']['password']
                                                     ,config['dbs']['dwh_177_public']['host']
                                                    ,config['dbs']['dwh_177_public']['port']
                                                    ,config['dbs']['dwh_177_public']['dbname']))
    cur = conn.cursor()
    sql = """DELETE FROM public.""" + tablename_pop +""" WHERE month = '""" + month + """';"""
    cur.execute(sql)
    conn.commit()
    engine = create_engine("postgresql://%s:%s@%s:%s/%s"% (config['dbs']['dwh_177_public']['user']
                                                      ,config['dbs']['dwh_177_public']['password']
                                                     ,config['dbs']['dwh_177_public']['host']
                                                    ,config['dbs']['dwh_177_public']['port']
                                                    ,config['dbs']['dwh_177_public']['dbname']))
    lst_data_single[lst_data_single.month==month].to_sql(tablename_pop, engine, if_exists='append', index=False, schema='public')

def process_dataimport(date, config: dict = infra_analytics_config):
    if (int(date[5:7])>=3)&(int(date[5:7])<9):
        kydautu = '2H'+str(int(date[:4]))
    else:
        if (int(date[5:7])>=1)&(int(date[5:7])<3):
            kydautu= '1H'+str(int(date[:4]))
        else:
            kydautu= '1H'+str(int(date[:4])+1)
    year=kydautu[2:]+'-01-01'
    if not config:
        config_file = "./config.yaml"
        config = get_config(config_file)
    
    process_get_star_pop(date,config)
    process_dansonam(year,config)
    process_hodannam(year,config)
    process_thanhphandansonam(year,config)
    process_nganhangnam(year,config)
    process_chitieunam(year,config)
    process_benhviennam(year,config)
    process_trung_tam_y_tenam(year,config)
    process_truong_tieu_hocnam(year,config)
    process_truong_trung_hoc_co_sonam(year,config)
    process_truong_trung_hoc_pho_thongnam(year,config)
    process_truong_dai_hocnam(year,config)
    process_truong_cao_dangnam(year,config)
    process_chonam(year,config)
    process_sieu_thinam(year,config)
    process_trung_tam_thuong_mainam(year,config)
    process_doanh_nghiepnam(year,config)
    process_doanh_nghiep_vua_va_nhonam(year,config)
    process_doanh_nghiep_tu_nhannam(year,config)
    process_khach_sannam(year,config)
    process_khach_san_tu_nhannam(year,config)
    process_doanh_nghiep_co_von_nuoc_ngoainam(year,config)
    process_doanh_nghiep_co_hoat_dong_xuat_nhap_khaunam(year,config)
    process_doanh_nghiep_cong_nghe_thong_tinnam(year,config)
    process_doanh_nghiep_co_trang_thong_tin_dien_tunam(year,config)


import shlex
import json
import subprocess
import json
import html
import psycopg2 as pg
from sqlalchemy import create_engine
from psycopg2.extras import execute_values
from multiprocessing import Process, Queue, Array, current_process
from underthesea import text_normalize
from bs4 import BeautifulSoup
import requests
import unidecode
import os
# os.environ['NO_PROXY'] = "api.cellphones.com.vn"
# os.environ['NO_PROXY'] = "www.dienmayxanh.com"
import numpy as np
import pandas as pd
import re
import urllib.request as urllib
import warnings
warnings.filterwarnings('ignore')
os.environ['http_proxy'] = "http://proxy.hcm.fpt.vn:80"
os.environ['https_proxy'] = "http://proxy.hcm.fpt.vn:80"
import yaml
from .config import infra_analytics_config

def get_config(config_file):
    with open(config_file, "r") as cf:
        config = yaml.load(cf, Loader=yaml.SafeLoader)
    return config

def crawler_thegioididong(config: dict = infra_analytics_config):
    """
        + Load thông tin phường/xã toàn quốc từ /mnt/projects-data/phat_trien_ha_tang/file_static/vietnam_phuongxa.xls
        + Crawler thông tin shop thế giới di động theo tỉnh từ https://www.thegioididong.com/sieu-thi-the-gioi-di-dong
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
        + Chuẩn hoá thông tin các cột  tỉnh, quận/huyện, phường/xã, đường 
    """
    if not config: 
        config_file = "./config.yaml"
        config = get_config(config_file)
    xaphuong = pd.read_excel(config['crawler_data']['address_path'], sheet_name="Sheet1")
    xaphuong = xaphuong.rename(columns = {
        "Mã": "code",
        "Tên": "name",
        "Cấp": "level",
        "Quận Huyện": "district",
        "Tỉnh / Thành Phố": "province",
    })
    xaphuong.replace({'province' : { r'(Thành phố[ ])': '', r'Tỉnh[ ]': ''}}, regex=True,inplace=True)
    xaphuong['province'] = xaphuong['province'].apply(lambda x: unidecode.unidecode(str(x)))
    xaphuong['province'] = xaphuong['province'].str.lower()
    xaphuong.replace({'province' : {r'[ ]-[ ]':' ', r' ': '-'}}, regex=True,inplace=True)
    link = config['crawler_data']['link_thegioididong']
    df = []
    for row in xaphuong.province.unique():
        url = link + str(row)
        # print('--- url: {}'.format(url))
        hdr = {'User-Agent': 'Mozilla/5.0'}
        try:
            req = urllib.Request(url,headers=hdr)
            page = urllib.urlopen(req)
            soup = BeautifulSoup(page)
            new_feeds = soup.find('div', class_='storeaddress', on_click=False).findChildren("a", attrs={'class': not('href')}) 
            for feed in new_feeds:
                news = feed.getText(("!href")).replace("\n","").strip()
                # print('Text: {}'.format(news))
                df.append(news)
        except:
            break
    df_tgdd = pd.DataFrame({'list_address':df})
    df_tgdd['type'] = np.where(df_tgdd['list_address'].str.contains('Điện máy xanh'),
                          'Điện máy xanh','Thế giới di động')
    df_tgdd.replace({'list_address' : {r'Điện máy xanh[ ]':'', r'Thế giới di động[ ]': ''}}, regex=True,inplace=True)
    df_tgdd['list_address'] = df_tgdd['list_address'].apply(lambda x: re.sub("[(].*?[)]|[!][href]|[refref]|",'',x, flags=re.DOTALL))
    df_tgdd['province'] = df_tgdd['list_address'].str.split(',').str[-1]
    df_tgdd['district'] = df_tgdd['list_address'].str.split(',').str[-2]
    df_tgdd['ward'] = df_tgdd['list_address'].str.split(',').str[-3]
    df_tgdd['street'] = df_tgdd['list_address'].str.split(',').str[:-3]
    df_tgdd['street'] = [','.join(map(str, l)) for l in df_tgdd['street']]
    df_tgdd.drop('list_address',axis=1, inplace=True)
    
    df_tgdd['province']= df_tgdd['province'].apply(lambda x: str(x).strip())
    df_tgdd['district']= df_tgdd['district'].apply(lambda x: str(x).strip())
    df_tgdd['ward']= df_tgdd['ward'].apply(lambda x: str(x).strip())
    df_tgdd['street']= df_tgdd['street'].apply(lambda x: str(x).strip())
    df_tgdd['province'] = df_tgdd['province'].apply(lambda x :re.sub('\.$', '', x))
    df_tgdd.columns = ['store', 'province', 'district', 'ward', 'addr']
    return df_tgdd
def crawler_dienmaycholon(config: dict = infra_analytics_config):
    """
        + Crawler thông tin shop điện máy chợ lớn từ https://dienmaycholon.vn/he-thong-sieu-thi
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
    """
    if not config:
        config_file = "./config.yaml"
        config = get_config(config_file)
    link = config['crawler_data']['link_dienmaycholon']
    df_dmcl = pd.DataFrame(columns=['addr','lat','long'])
    try:
        hdr = {'User-Agent': 'Mozilla/5.0'}
        req = urllib.Request(link,headers=hdr)
        page = urllib.urlopen(req)
        soup = BeautifulSoup(page)
        new_feeds = soup.find('div', class_='position_market_parent').findAll("div", 
                                            attrs={'class': 'container_cart'}) 
        for feed in new_feeds:
            news = feed.getText(("")).replace('(Xem bản đồ)','')
            link_ = str("https://dienmaycholon.vn")+feed.find("a").get("href")
            req_ = urllib.Request(link_,headers=hdr)
            page_ = urllib.urlopen(req_)
            soup_ = BeautifulSoup(page_)
            lat = soup_.find('div', id='map')['lat']
            long = soup_.find('div', id='map')['lng']
        #     print(news)
            df2 = pd.DataFrame({"addr":[news],
                            "lat":[float(lat)],
                            "long":[float(long)]})
            df_dmcl= df_dmcl.append(df2, ignore_index = True)
    except:
        print('lỗi điện máy chợ lớn')
    df_dmcl['store'] = 'Điện máy chợ lớn'
    df_dmcl['addr'] = df_dmcl['addr'].apply(lambda x: re.sub("[(].*?[)]",'',x, flags=re.DOTALL))
    df_dmcl['province'] = df_dmcl['addr'].str.split(',').str[-1]
    df_dmcl['district'] = df_dmcl['addr'].str.split(',').str[-2]
    df_dmcl['ward'] = df_dmcl['addr'].str.split(',').str[-3]
    df_dmcl['street'] = df_dmcl['addr'].str.split(',').str[:-3]
    df_dmcl['street'] = [','.join(map(str, l)) for l in df_dmcl['street']]
    return df_dmcl
def crawler_dienmayhc(config: dict = infra_analytics_config):
    """
        + Crawler thông tin shop điện máy HC từ https://hc.com.vn/ords/s--he-thong-sieu-thi-dien-may-hc
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
    """
    if not config:
        config_file = "./config.yaml"
        config = get_config(config_file)
    link_hc = config['crawler_data']['link_dienmayhc']
    df_dmhc = pd.DataFrame(columns=['addr','lat','long'])
    try:
        hdr = {'User-Agent': 'Mozilla/5.0'}
        req = urllib.Request(link_hc, headers=hdr)
        page = urllib.urlopen(req)
        soup = BeautifulSoup(page)
        new_feeds = soup.find('div', class_='col-xs-12 welcome-about-us').findAll('p')
        for feed in new_feeds:
            news = feed.getText(("")).replace("Xem bản đồ","").replace("-","")
            news= list(news.split("\n"))
        #     print(news)
            for i in range(len(feed.find_all('a', href=True))):
                a= feed.find_all('a', href=True)[i]
                req_ = urllib.Request(a['href'],headers=hdr)
                page_ = urllib.urlopen(req_)
                soup_ = BeautifulSoup(page_)
                td = soup_.find('div', class_='col-xs-12 welcome-about-us')
                # print(news[i])
                if (td is not None):
                    td = td.find('a').get('href')
                    m = re.match(r".*@(?P<lat>\d+\.?\d+),(?P<long>\d+\.?\d+)", td)
                    df2 = pd.DataFrame({"addr":[news[i]]})
                    df_dmhc= df_dmhc.append(df2, ignore_index = True)
                else:
                    df2 = pd.DataFrame({"addr":[news[i]],
                        "lat":[None],
                        "long":[None]})
                    df_dmhc= df_dmhc.append(df2, ignore_index = True)
    except:
        print('lỗi điện máy hc')
    # df_dmcl.replace({'addr' : { r'(Khai trương)': '', r'Mới khai trương': ''}}, regex=True,inplace=True)
    df_dmhc['store'] = 'Điện máy hc'
    df_dmhc['addr'] = df_dmhc['addr'].str.split('Khai trương').str[0]
    df_dmhc['addr'] = df_dmhc['addr'].str.split('Mới khai trương').str[0]
    df_dmhc['addr'] = df_dmhc['addr'].str.split(':').str[1]
    df_dmhc['province'] = df_dmhc['addr'].str.split(',').str[-1]
    df_dmhc['district'] = df_dmhc['addr'].str.split(',').str[-2]
    df_dmhc['ward'] = df_dmhc['addr'].str.split(',').str[-3]
    df_dmhc['street'] = df_dmhc['addr'].str.split(',').str[:-3]
    df_dmhc['street'] = [','.join(map(str, l)) for l in df_dmhc['street']]
    return df_dmhc
def crawler_dienmaynguyenkim():
    """
        + Crawler thông tin shop điện máy Nguyễn Kim từ https://www.nguyenkim.com/index.php?dispatch=nk_mp_mall.apimall
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
    """
    cURL = r"""curl 'https://www.nguyenkim.com/index.php?dispatch=nk_mp_mall.apimall' \
          -H 'Accept: application/json, text/javascript, */*; q=0.01' \
          -H 'Accept-Language: vi-VN,vi;q=0.9,fr-FR;q=0.8,fr;q=0.7,en-US;q=0.6,en;q=0.5' \
          -H 'Connection: keep-alive' \
          -H 'Cookie: __uzma=c8ed7501-db2b-46ff-96b2-861f558fb77a; __uzmb=1701424464; __uzme=8643; _gid=GA1.2.105846784.1701424465; __ssds=2; __rtbh.lid=%7B%22eventType%22%3A%22lid%22%2C%22id%22%3A%22RS1ALAW0DTWVTAA7h73Q%22%7D; _atm_objs=eyJzb3VyY2UiOiJjaXR5YWRzIiwibWVkaXVtIjoiY3BhIiwiY2FtcGFpZ24iOiIyTktaIiwiY29u%0D%0AdGVudCI6IiIsInRlcm0iOiIiLCJ0eXBlIjoiYXNzb2NpYXRlX3V0bSIsImNoZWNrc3VtIjoiKiIs%0D%0AInRpbWUiOjE3MDE0MjQ0NjUwMDV9; state_name=TP.HCM; state_code=001; unauthHomeLocation_code=001; storeLocation_name=TP.HCM; storeLocation_code=001; installmentLocation_name=TP.HCM; installmentLocation_code=001; __ssuzjsr2=a9be0cd8e; __uzmaj2=0974ffd7-398a-46f8-b717-7f6fd25d2d3d; __uzmbj2=1701424465; _ac_au_gt=1701424465226; login_form_event=sign_in; mp_skin=desktop; _fbp=fb.1.1701424465546.141731762; utm_source=cityads; utm_medium=cpa; utm_campaign=2NKZ; click_id=8nNZ20RSfjZt9V4; sid_customer_5120c=57dd406f9c53054e0013f495114b9ce4-C; nk_auth=unauth; _asm_uid=1297867988; _ac_client_id=1297876943.1701424472; au_id=1297876943; __zi=2000.SSZzejyD5ja-a_QhmHqUcJQLzgUEK0FNE8sq-vnQ7SLwdwBesmLRndZJw-_V15d7Sj6Wwj0F4iDtqQFdr0OVpG.1; nkcache_id=22b447c26392cdeead3b3a1f0d8a02f5; _asm_visitor_type=r; login_form_event_time=1701483785; _utm_objs=eyJzb3VyY2UiOiJyLnNydnRyY2suY29tIiwibWVkaXVtIjoicmVmZXJyZXIiLCJjYW1wYWlnbiI6%0D%0AIiIsImNvbnRlbnQiOiIiLCJ0ZXJtIjoiIiwidHlwZSI6InJlZmVycmVyIiwidGltZSI6MTcwMTQ4%0D%0ANDE1NTY5MiwiY2hlY2tzdW0iOiJjaTV6Y25aMGNtTnJMbU52YlMxeVpXWmxjbkpsY2kwdE1UY3dN%0D%0AVFE0TkRFMU5UWTVNZz09In0%3D; _gat_UA-17048930-1=1; cto_bundle=W1o-gl8lMkZQeDFpd0RDdWtvRVA2cDY3SiUyRlBJRFRaT0tZSHVScE14RGxKJTJGVkRMRkdXWGUyWEdFUk5JdFcwS1NaYjhhdVhsNmVpTGJMWWpxSW5CZlRjSVFzR3RGY3Z5M3Y5VXRrT0JoMWRiTk9yMlpuOHJkWWdkZlY3bDRQQlhWJTJCbEg2Nk9rZHNiR2lEQk5vcHFQQm1oZEJhOUFuRUNzQzdwVWx5JTJCdlVhN3prVjBnUFYlMkJzSHE2aElDJTJGSmczWDM4TiUyRm5HU0s2NGJaNGI3QjhaclJpN2sxZVpCb3ZXazlENmdqYlNCaGJxTVllUkQyT1A1TVpOME03ME5tNmsxdXoyUnA1MzFyVVlnY1FsZHNWUkFZQlRTRU8wVlV6ZjlQY0N6N1BGWG5veXVIZDdZaG9JcW9DVXVQZG12WGpJNXY0UkJSaldYbXdpMjRwcmpaU1dDQUVnJTJCZmpneEhQanJVdGVQcm1EJTJGZ1BHOXNvUWFrMXdoRSUzRA; _pk_ref.554926188.973b=%5B%22%22%2C%22%22%2C1701505007%2C%22https%3A%2F%2Fexcel.officeapps.live.com%2F%22%5D; _pk_id.554926188.973b=1297876943.1701424465.3.1701505007.1701505007.; _pk_ses.554926188.973b=*; __uzmcj2=729563435905; __uzmdj2=1701505007; _clck=1n3tsu%7C2%7Cfh7%7C0%7C1430; _cdp_cfg=%257B%2522refferal_exclusion%2522%3A%255B%2522secureacceptance.cybersource.com%2522%2C%2522nguyenkim.com%2522%255D%257D; _ac_an_session=zkzhznzqzizlzmzlzjzgzrzkzmzkzkzizdzjzdzizkzjzizmzjzmzjzjzkzdzizdzizkzjzizmzjzmzjzjzkzdzizkzjzizmzjzmzjzjzkzdzizdzhzqzdzizd2f27zdzgzdzlzmzmznzqzd; _clsk=1v2k3ka%7C1701505008331%7C1%7C1%7Co.clarity.ms%2Fcollect; _ga_8S8EFGF74J=GS1.2.1701505007.3.0.1701505009.58.0.0; _gat=1; _ga=GA1.1.576348471.1701424465; __uzmc=3718810948761; __uzmd=1701505042; _ga_34B604LFFQ=GS1.1.1701505006.3.1.1701505042.24.0.0' \
          -H 'Referer: https://www.nguyenkim.com/cac-trung-tam-mua-sam-nguyen-kim.html' \
          -H 'Sec-Fetch-Dest: empty' \
          -H 'Sec-Fetch-Mode: cors' \
          -H 'Sec-Fetch-Site: same-origin' \
          -H 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36' \
          -H 'X-Requested-With: XMLHttpRequest' \
          -H 'sec-ch-ua: "Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24"' \
          -H 'sec-ch-ua-mobile: ?0' \
          -H 'sec-ch-ua-platform: "macOS"' \
          --compressed"""
    df_nk = pd.DataFrame(columns=['address'])
    try:
        lCmd = shlex.split(cURL) # Splits cURL into an array
        p = subprocess.Popen(lCmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate() # Get the output and the err message
        json_data = json.loads(out.decode("utf-8"))
        lst_data_single = pd.DataFrame(json_data)
        df_nk = lst_data_single[['address']]
    except:
        print('không có dữ liệu')
    df_nk['store'] ='Nguyễn Kim'
    df_nk['province'] = df_nk['address'].str.split(',').str[-1]
    df_nk['district'] = df_nk['address'].str.split(',').str[-2]
    df_nk['ward'] = df_nk['address'].str.split(',').str[-3]
    df_nk['street'] = df_nk['address'].str.split(',').str[:-3]
    df_nk['street'] = [','.join(map(str, l)) for l in df_nk['street']]
    df_nk.columns = ['addr', 'store', 'province', 'district', 'ward', 'street']
        
    return df_nk
def crawler_cellphone():
    """
        + Crawler thông tin shop điện máy cellphone từ https://api.cellphones.com.vn/graphql-dashboard/graphql/query
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
    """
    cURL = r"""curl 'https://api.cellphones.com.vn/graphql-dashboard/graphql/query' \
      -H 'authority: api.cellphones.com.vn' \
      -H 'accept: application/json' \
      -H 'accept-language: vi-VN,vi;q=0.9,fr-FR;q=0.8,fr;q=0.7,en-US;q=0.6,en;q=0.5' \
      -H 'content-type: application/json' \
      -H 'origin: https://cellphones.com.vn' \
      -H 'referer: https://cellphones.com.vn/' \
      -H 'sec-ch-ua: "Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24"' \
      -H 'sec-ch-ua-mobile: ?0' \
      -H 'sec-ch-ua-platform: "macOS"' \
      -H 'sec-fetch-dest: empty' \
      -H 'sec-fetch-mode: cors' \
      -H 'sec-fetch-site: same-site' \
      -H 'user-agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36' \
      --data '{"query":"query getAllStores{\n                          GetDataMap4d(lat : 10.7914333,long : 106.687859){\n                              shop {\n                                id\n                                code\n                                external_id\n                                address\n                                phone\n                                google_link\n                                zalo_link\n                                district_id\n                                province_id\n                                latitude\n                                longitude\n                                time_opening\n                                shop_description\n                                google_map_address\n                                company_id\n                                near\n                                created_at\n                                deleted_at\n                                updated_at\n                                store_opening_images\n                                store_opening_url\n                              }\n                              location {\n                                status\n                                distance {\n                                    text\n                                    value\n                                }\n                                duration {\n                                    text\n                                    value\n                                }\n                              }\n                              cps\n                              asp\n                              dtv\n                              slug\n                          }\n                        }","variables":{}}' \
      --compressed"""
    
    lCmd = shlex.split(cURL) # Splits cURL into an array
    
    p = subprocess.Popen(lCmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate() # Get the output and the err message
    json_data = json.loads(out.decode("utf-8"))
    lst_data_single = pd.DataFrame(json_data['data']['GetDataMap4d'])
    json_struct = json.loads(lst_data_single.to_json(orient="records"))    
    
    df_cp_full = pd.io.json.json_normalize(json_struct)
    df_cp = df_cp_full[['shop.address','slug']]
    df_cp.columns=['address','province']
    df_cp['store'] ='CellphoneS'
    df_cp.replace({'province':{r'-':' '}}, regex=True,inplace=True)
    df_cp['province'] = df_cp['province'].str.title()
    df_cp['district'] = df_cp['address'].str.split(',').str[-1]
    df_cp['ward'] = df_cp['address'].str.split(',').str[-2]
    df_cp['street'] = df_cp['address'].str.split(',').str[:-2]
    df_cp['street'] = [','.join(map(str, l)) for l in df_cp['street']]
    df_cp.columns = ['addr', 'province', 'store', 'district', 'ward', 'street']
    return df_cp
def crawler_dienmayxanh():
    """
        + Crawler thông tin shop điện máy cellphone từ https://www.dienmayxanh.com/Store/SearchStoreByValue
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
    """
    df_dmx = pd.DataFrame(columns=['addr']) 
    try:
        for i in range(321):
            cURL = r"""curl 'https://www.dienmayxanh.com/Store/SearchStoreByValue' \
                  -H 'Accept: */*' \
                  -H 'Content-Type: application/x-www-form-urlencoded; charset=UTF-8' \
                  -H 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36' \
                  -H 'X-Requested-With: XMLHttpRequest' \
                  --data 'pageIndex={}'""".format(i)
                
            lCmd = shlex.split(cURL) # Splits cURL into an array
            p = subprocess.Popen(lCmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = p.communicate() # Get the output and the err message
            # json_data = json.loads(out.decode("utf-8"), strict=False)
            try:
                json_data = json.loads(out.decode("utf-8"), strict=False)
            except json.JSONDecodeError as e:
                if "Expecting value" in str(e):
                    # Handle empty content
                    json_data = None
                else:
                    # Handle other JSONDecodeError
                    raise
            decoded_string = html.unescape(json_data['html'])
            soup = BeautifulSoup(decoded_string, 'html.parser')
            new_feeds = soup.findAll('span')
            for feed in new_feeds:
                news = feed.getText((""))
                df2 = pd.DataFrame({"addr":[news]})
                df_dmx= df_dmx.append(df2, ignore_index = True)
    except:
        print('lỗi điện máy')
    df_dmx.replace({'addr':{'\n|Điện máy Xanh|Thế giới di động|, Việt Nam|- Xem bản đồ':''}},regex=True,inplace=True)
    df_dmx['addr'] = df_dmx['addr'].apply(lambda x: re.sub("[(].*?[)]|[!][href]|[refref]|[)]|[|]",'',x,
                                                                               flags=re.DOTALL))
    df_dmx['province'] = df_dmx['addr'].str.split(',').str[-1]
    df_dmx['district'] = df_dmx['addr'].str.split(',').str[-2]
    df_dmx['ward'] = df_dmx['addr'].str.split(',').str[-3]
    df_dmx['street'] = df_dmx['addr'].str.split(',').str[:-3]
    df_dmx['street'] = [','.join(map(str, l)) for l in df_dmx['street']]
    df_dmx_fil = df_dmx[['addr','province', 'district', 'ward', 'street']]
    df_dmx_fil['province']= df_dmx_fil['province'].apply(lambda x: str(x).strip())
    df_dmx_fil['district']= df_dmx_fil['district'].apply(lambda x: str(x).strip())
    df_dmx_fil['ward']= df_dmx_fil['ward'].apply(lambda x: str(x).strip())
    df_dmx_fil['street']= df_dmx_fil['street'].apply(lambda x: str(x).strip())
    df_dmx_fil['province'] = df_dmx_fil['province'].apply(lambda x :re.sub('\.$', '', x))
    df_dmx_fil['province'] = df_dmx_fil['province'].apply(lambda x :re.sub('\s\s+', ' ', x))
    df_dmx_fil['store'] ='Điện máy xanh'
    return df_dmx_fil
def crawler_fshop():
    """
        + Crawler thông tin shop điện máy cellphone từ https://fptshop.com.vn/cua-hang/Home/GetListShop?page={}
        + Tách thông tin tỉnh, quận/huyện, phường/xã, đường từ thông tin address crawler được 
    """
    df_fshop = pd.DataFrame(columns=['addr'])
    try:
        for i in range(1,40):
            cURL = r"""curl 'https://fptshop.com.vn/cua-hang/Home/GetListShop?page={}' \
              -H 'Accept: */*' \
              -H 'Accept-Language: vi-VN,vi;q=0.9,fr-FR;q=0.8,fr;q=0.7,en-US;q=0.6,en;q=0.5' \
              -H 'Connection: keep-alive' \
              -H 'Cookie: log_6dd5cf4a-73f7-4a79-b6d6-b686d28583fc=defb10eb-2054-43d1-9767-c8ed78ba803b; __zi=2000.SSZzejyD7iu_cVEzsr0LpYAPvhoKKa7GR9V-_iX0Iyv-rUpesmSKY7tNeA7S0nMAVDsYwjaC7i5usABfr0WNp3O.1; cf_clearance=EjDwVUTyNaWTqtTdiz3QZb6dOYVJBrqWBOUQRNjaUeY-1701506110-0-1-9288697f.9912e224.c589369f-0.2.1701506110; _gid=GA1.3.1578401529.1701506111; _gat=1; _ga=GA1.1.1244735801.1701413780; _gcl_au=1.1.1178009718.1701515146; ajs_group_id=null; fpt_uuid=%221fa9f1ac-d03d-4fae-8610-daf69ea27add%22; __rtbh.lid=%7B%22eventType%22%3A%22lid%22%2C%22id%22%3A%22CPuCgjmaMeNWKnRndpnx%22%7D; _fbp=fb.2.1701515146558.1614252460; __admUTMtime=1701515146; _tt_enable_cookie=1; _ttp=shMRdTnH6HtY_0plx4pRPytFZlG; dtdz=83a9a83f-2cf0-4b77-a74e-5f68bd07d459; __iid=; __iid=; __su=0; __su=0; _hjSessionUser_731679=eyJpZCI6IjhkYTAwY2E5LTNiY2MtNTI0Yy05YjNmLWM4NjNiN2UxNThkZSIsImNyZWF0ZWQiOjE3MDE1MTUxNDg1MTgsImV4aXN0aW5nIjpmYWxzZX0=; _hjFirstSeen=1; _hjIncludedInSessionSample_731679=0; _hjSession_731679=eyJpZCI6ImQwMGJjNjAzLWI1ZDgtNGRlMC04OTBlLTlhNTNjMjY2ODE4NSIsImNyZWF0ZWQiOjE3MDE1MTUxNDg1MjAsImluU2FtcGxlIjpmYWxzZSwic2Vzc2lvbml6ZXJCZXRhRW5hYmxlZCI6dHJ1ZX0=; _hjAbsoluteSessionInProgress=1; __RC=5; __R=3; __uif=__uid%3A1897444670985365354%7C__ui%3A2%252C5%7C__create%3A1697444670; ins-click_tra_gop=true; _ga_34B604LFFQ=GS1.1.1701515145.3.1.1701515150.55.0.0; _ga_ZR815NQ85K=GS1.1.1701515145.3.1.1701515151.54.0.0' \
              -H 'Referer: https://fptshop.com.vn/cua-hang' \
              -H 'Sec-Fetch-Dest: empty' \
              -H 'Sec-Fetch-Mode: cors' \
              -H 'Sec-Fetch-Site: same-origin' \
              -H 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36' \
              -H 'X-Requested-With: XMLHttpRequest' \
              -H 'sec-ch-ua: "Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24"' \
              -H 'sec-ch-ua-mobile: ?0' \
              -H 'sec-ch-ua-platform: "macOS"' \
              --compressed""".format(i)
            lCmd = shlex.split(cURL) # Splits cURL into an array
            
            p = subprocess.Popen(lCmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = p.communicate() # Get the output and the err message
            json_data = json.loads(out.decode("utf-8"))
            soup = BeautifulSoup(json_data['view'], 'html.parser')
            if soup.findAll('h2', class_='label-text f-w-400')!=[]:
                new_feeds = soup.findAll('h2', class_='label-text f-w-400')
                
            elif (soup.findAll('span', class_='label-text f-w-400')!=[]):
                new_feeds = soup.findAll('span', class_='label-text f-w-400')
            else:
                new_feeds = soup.findAll('div', class_='label-text f-w-400')
            for feed in new_feeds:
                news = feed.getText((""))
                df2 = pd.DataFrame({"addr":[news]})
                df_fshop= df_fshop.append(df2, ignore_index = True)
    except:
        print('lỗi')
    df_fshop['store'] ='Fshop'
    df_fshop['province'] = df_fshop['addr'].str.split(',').str[-1]
    df_fshop['district'] = df_fshop['addr'].str.split(',').str[-2]
    df_fshop['ward'] = df_fshop['addr'].str.split(',').str[-3]
    df_fshop['street'] = df_fshop['addr'].str.split(',').str[:-3]
    return df_fshop
def infra_crawlershopdienmay(config: dict = infra_analytics_config):
    """
        + Call các function crawler shop điện máy trên và concat thành dataframe 
        + Drop các row duplicate 
        + Upsert thông tin shop điện máy vào postgresql 177 - dwh_noc - public.tbl_shop_info
    """
    if not config:
        config_file = "./config.yaml"
        config = get_config(config_file)
    
    df_nk=crawler_dienmaynguyenkim()
    df_fshop = crawler_fshop()
    df_hc=crawler_dienmayhc()
    df_cl=crawler_dienmaycholon()
    df_tgdd=crawler_thegioididong()
    df_dmx=crawler_dienmayxanh()
    df_cp=crawler_cellphone()
    
    df_fshop =  df_fshop[['addr','store','province','district','ward']]
    df_dmx = df_dmx[['addr','store','province','district','ward']]
    df_cp = df_cp[['addr','store','province','district','ward']]
    df_nk = df_nk[['addr','store','province','district','ward']]
    df_hc = df_hc[['addr','store','province','district','ward']]
    df_cl = df_cl[['addr','store','province','district','ward']]
    df_tgdd = df_tgdd[['addr','store','province','district','ward']]
    df_full = pd.concat([df_fshop,df_dmx,df_cp,df_nk,df_hc,df_cl,df_tgdd])
    df_full['addr'] = np.where(df_full['store']=='Thế giới di động', df_full['addr']+','+df_full['ward']+','
                               +df_full['district']+','+df_full['province'],df_full['addr'])
    df_full.drop_duplicates(keep='first',inplace=True)
    lst_data = df_full.values.tolist()
    conn = pg.connect("postgresql://%s:%s@%s:%s/%s"% (config['dbs']['dwh_177_public']['user']
                                                      ,config['dbs']['dwh_177_public']['password']
                                                     ,config['dbs']['dwh_177_public']['host']
                                                    ,config['dbs']['dwh_177_public']['port']
                                                    ,config['dbs']['dwh_177_public']['dbname']))
    cur_1 = conn.cursor()
    for i in range(len(lst_data)):
        tupl = (lst_data[i])
        cur_1.execute(
        " INSERT INTO tbl_shop_info(addr,store,province,district, ward) VALUES( %s, %s, %s, %s, %s)"
        " ON CONFLICT (addr)"
        " DO UPDATE SET store = EXCLUDED.store, "
        " province = EXCLUDED.province, district = EXCLUDED.district, ward = EXCLUDED.ward;", (tuple(tupl))
        )
    try:
        conn.commit()
        conn.close()
        print("Successfully!!!!")
    except:
        print("Don't save DB")



import spark_sdk as ss
ss.__version__
import os
ss.PySpark(yarn=False, num_executors=60, driver_memory='60g', executor_memory='24g',
            add_on_config1=('spark.port.maxRetries', '1000'))
spark = ss.PySpark().spark
from pyspark.sql.types import DoubleType, IntegerType
spark.sql("SET spark.sql.sources.partitionOverwriteMode = dynamic")
from pyspark.sql.window import Window
from pyspark.sql.functions import row_number,col

from datetime import timedelta
from datetime import datetime
from dateutil.relativedelta import relativedelta
import pandas as pd
import numpy as np
import glob, re
import pyspark.pandas as ps
import sh
import pyspark.sql.functions as f
import sqlalchemy
import psycopg2 as pg
from .config import infra_analytics_config

import yaml
def get_config(config_file):
    with open(config_file, "r") as cf:
        config = yaml.load(cf, Loader=yaml.SafeLoader)
    return config
    
def extract_date(file_path):
    # Split tên file để lấy phần ngày tháng
    file_name = file_path.split('/')[-1]  # Lấy phần cuối cùng sau dấu '/'
    date_str = file_name.split('_')[-1]   # Lấy phần sau dấu '_' chứa ngày tháng
    date_str = date_str.replace('.xlsx','')
    # Chuyển đổi từ string sang datetime
    return datetime.strptime(date_str, '%Y%m%d')

def process_perport(kydautu, config: dict = infra_analytics_config):
    """
    Tổ chức lưu trữ dữ liệu về chi phí đầu tư của các kế hoạch theo kỳ:
       + Load data import: /data/fpt/ftel/infra/dwh/import_data_phattrienhatang/chi_phi_dau_tu
       + Chuyển đổi kiểu dữ liệu cho đồng bộ schema lưu trữ trước đó
       + Lưu data xuống hdfs:/data/fpt/ftel/infra/dwh/phat_trien_ha_tang/chi_phi_dau_tu.parquet 
       + Groupby theo mã kế hoạch tính trung bình chi phí đầu tư mỗi port 
    """
    if not config:
        config_file = "./config.yaml"
        config = get_config(config_file)
    hdfsdir = config['data_import']['source_path']
    filelist = [ line.rsplit(None,1)[-1] for line in sh.hdfs('dfs','-ls',hdfsdir).split('\n') if len(line.rsplit(None,1))][1:]
    keyword = config['data_import']['chiphidautu_path_input']
    # Initialize an empty list to store matching file paths
    matching_files = []
    
    # Iterate through the file list
    for file_path in filelist:
        # Check if the keyword is present in the file path
        if keyword in file_path:
            # If yes, append the file path to the matching_files list
            matching_files.append(file_path)
    # Lấy ra file có ngày tạo gần đây nhất
    recent_file = max(matching_files, key=extract_date)
    print(recent_file)
    df= ps.read_excel(recent_file,dtype=str)
    df = df.rename({'DL triển khai':'Dung lượng triển khai',
               'Phường':'Phường/Xã',
               'Quận':'Quận/Huyện',
               'Ngày hoàn tất thi công':'Ngày hoàn tất'}, axis=1)
    df = df[['Kỳ đầu tư','Vùng','Chi nhánh','Loại KH','POP','Dung lượng triển khai',
        'Phường/Xã','Quận/Huyện','Mã kế hoạch','Tổng chi phí','Perport','Ngày hoàn tất']]
    df.columns = ['ky_dau_tu', 'Vùng', 'Chi nhánh', 'Loại KH', 'POP',
           'Dung lượng triển khai', 'Phường/Xã', 'Quận/Huyện', 'ma_ke_hoach',
           'Tổng chi phí', 'perport', 'Ngày hoàn tất thi công']
    
    df_grp = df.groupby('ky_dau_tu',as_index=False).agg({'ma_ke_hoach':'count'})
    kydautu_max = df_grp[df_grp.ma_ke_hoach==df_grp.ma_ke_hoach.max()]['ky_dau_tu'][0]
    df_w = df[df.ky_dau_tu==kydautu_max]
    
    df_ = spark.createDataFrame(df_w.to_pandas())
    df_ = df_.withColumn("Dung lượng triển khai", df_["Dung lượng triển khai"].cast(IntegerType()).alias("Dung lượng triển khai"))
    df_ = df_.withColumn("Tổng chi phí", df_["Tổng chi phí"].cast(DoubleType()).alias("Tổng chi phí"))
    df_ = df_.withColumn("perport", df_["perport"].cast(DoubleType()).alias("perport"))
    
    df_ = df_.withColumn('kydautu', col('ky_dau_tu'))
    df_.coalesce(1)\
        .write.mode("overwrite")\
        .partitionBy('kydautu')\
        .option("path", config['data_import']['chiphidautu_path_output'])\
        .save()

    df_dt_perport = df[df.ky_dau_tu==kydautu].groupby(['ma_ke_hoach'],as_index=False).agg({'perport':'mean'})
    df_dt_perport['check_kh']=1
    return df_dt_perport
    
def tbl_planning_history(date, config: dict = infra_analytics_config):
    """
    Upsert hiệu quả khai thác của các kế hoạch theo thời gian:
       + Load data từ /data/fpt/ftel/infra/dwh/infor_port_monthly.parquet
       + Xử lý lấy ngày thi công từ mã kế hoạch => Nếu createdate null thì gán bằng ngày thi công
       + Xử lý lấy ngày used_date là ngày createdate sớm nhất (nếu có nhiều createdate)
       + Tính tuổi tập điểm: date (ngày ghi nhận dữ liệu) - used_date (ngày đưa vào sử dụng)
       + Tính delta_month (3T, 6T, 9T, 12T, > 1 năm): từ tuổi của tập điểm 
       + Lọc lấy thời điểm mà tập điểm có port cao nhất theo: 'Plans','region', 'branch','district','ward', 'popname', 'name',  'delta_month'
       + Groupby: 'Plans','ky_dau_tu', 'popname','region', 'branch','district','ward', 'delta_month' => Tính tổng port, portuse 
       + Pivot table để có các cột port theo delta_month
       + Mapping thêm chi phí đầu tư mỗi port từ function process_perport 
       + Tính tỉ lệ khai thác theo các khoảng thời gian: port_use/port
       + Thực hiện upsert vào bảng lưu trữ lịch sử đầu tư trước đó tbl_planning_history (server 177 - dwh_noc)
    """
    date = datetime.strptime(date, '%Y-%m-%d')
    if (date.month>=1)&(date.month<7):
        kydautu = '1H'+str(date.year)
    else:
        kydautu= '2H'+str(date.year)
    if not config:
        config_file = "./config.yaml"
        config = get_config(config_file)
    base_path = config['infor_port_monthly']['infor_port_monthly_path_output']
    df= spark.read.parquet(base_path).cache()
    df = df.withColumn('str_ngay_thi_cong', f.split(df['Plans'], '[.]').getItem(4))
    df = df.withColumn('str_ngay_thi_cong', f.when(f.col('str_ngay_thi_cong').contains('PP'), f.col('str_ngay_thi_cong').substr(-6, 6))\
         .otherwise(f.col('str_ngay_thi_cong')))
    df = df.withColumn('ngay_thi_cong', f.to_date(df["str_ngay_thi_cong"], "ddMMyy"))
    df = df.withColumn('createdate', f.when(df.createdate.isNull(),df.ngay_thi_cong).otherwise(df.createdate))
    df_drp = df.groupBy(['Plans','region','branch','district','ward','popname','name','port','portuse','portfree',
                'createdate', 'date']).agg(f.count(f.col('d')).alias('num_row')).cache()
    df_old =df.groupBy(['Plans','region','branch','district','ward','popname','name']).agg(f.min(f.col('createdate')).alias('used_date')).cache()
    df_full = df_drp.join(df_old, on=['Plans','region','branch','district','ward','popname','name'], how='left').cache()
    df_full= df_full.withColumn("month_old_device", f.round(f.months_between(f.col("date"),f.col("used_date")),0)).cache()
    df_full= df_full.withColumn("delta_month",f.when(df_full.month_old_device <=3, '3T')\
            .when((df_full.month_old_device > 3)&(df_full.month_old_device <=6), '6T')\
            .when((df_full.month_old_device > 6)&(df_full.month_old_device <=9), '9T')\
            .when((df_full.month_old_device > 9)&(df_full.month_old_device <=12), '12T').otherwise('>1 năm')).cache()
    df_full_prp = df_full.sort(f.col("Plans").desc(),f.col("region").desc(),f.col("branch").desc(),f.col("district").desc(),f.col("ward").desc()
        ,f.col("popname").desc(),f.col("name").desc(),f.col("delta_month").desc(),f.col("date").desc())
    df_full_prp = df_full_prp.dropDuplicates(['Plans','region', 'branch','district','ward', 'popname', 'name', 
                                              'delta_month']).cache()
    df_full_prp = df_full_prp.filter(df_full_prp.delta_month!='>1 năm')
    df_full_prp = df_full_prp.withColumn('ky_dau_tu',f.when((f.month(df_full_prp.createdate)>=1)&\
                            (f.month(df_full_prp.createdate)<7),f.concat(f.lit('1H'),f.lit(f.year(df_full_prp.createdate))))\
                            .otherwise(f.concat(f.lit('2H'),f.lit(f.year(df_full_prp.createdate)))))
    df_full_plans = df_full_prp.groupBy(['Plans','ky_dau_tu', 'popname','region', 'branch','district','ward',
                           'delta_month']).agg(f.sum('port').alias('port'),f.sum('portuse').alias('portuse'))
    df_port_pd = df_full_plans.toPandas()
    df_port_pd= df_port_pd.sort_values(['Plans','ky_dau_tu', 'popname','region', 'branch','district','ward',
                           'delta_month'],ascending=True)
    df_port_pd_filter = df_port_pd[df_port_pd.Plans!='']
    df_port_pd_filter.rename({'Plans':'ma_ke_hoach'},axis=1,inplace=True)
    df_port_pivot = pd.pivot_table(df_port_pd_filter, values=['port','portuse'], index=['ma_ke_hoach',
                'ky_dau_tu', 'popname', 'region', 'branch', 'district', 'ward'],
                         columns=['delta_month'],  aggfunc={"mean"}, fill_value=None).reset_index()
    df_port_pivot.columns=['ma_ke_hoach','ky_dau_tu', 'pop', 'vung', 'chi_nhanh'
    , 'quan', 'phuong','tong_port_sau_12t','tong_port_sau_3t',
    'tong_port_sau_6t','tong_port_sau_9t','port_dung_sau_12t','port_dung_sau_3t','port_dung_sau_6t'
    ,'port_dung_sau_9t']
    df_dt_perport = process_perport(kydautu)
    df_dt_perport = df_dt_perport.to_pandas()
    df_port_pivot = df_port_pivot.merge(df_dt_perport,on='ma_ke_hoach',how='left')
    df_port_pivot = df_port_pivot[(~df_port_pivot.check_kh.isna())&(~df_port_pivot.vung.isna())]
    df_port_pivot.drop('check_kh',axis=1,inplace=True)
    df_port_pivot['dung_luong_trien_khai'] = df_port_pivot['tong_port_sau_3t']
    df_port_pivot['ti_le_khai_thac_3t'] = np.where(df_port_pivot['tong_port_sau_12t']<=0,0,
                      df_port_pivot['port_dung_sau_3t']/df_port_pivot['tong_port_sau_12t'])
    df_port_pivot['ti_le_khai_thac_6t'] = np.where(df_port_pivot['tong_port_sau_6t']<=0,0,
                      df_port_pivot['port_dung_sau_6t']/df_port_pivot['tong_port_sau_6t'])
    df_port_pivot['ti_le_khai_thac_9t'] = np.where(df_port_pivot['tong_port_sau_9t']<=0,0,
                      df_port_pivot['port_dung_sau_9t']/df_port_pivot['tong_port_sau_9t'])
    df_port_pivot['ti_le_khai_thac_12t'] = np.where(df_port_pivot['tong_port_sau_12t']<=0,0,
                      df_port_pivot['port_dung_sau_12t']/df_port_pivot['tong_port_sau_12t'])
    df_port_pivot_filter = df_port_pivot[['ma_ke_hoach', 'ky_dau_tu', 'pop', 'phuong', 'quan', 'chi_nhanh',
           'vung', 'dung_luong_trien_khai', 'perport', 'tong_port_sau_3t',
           'port_dung_sau_3t', 'ti_le_khai_thac_3t', 'tong_port_sau_6t',
           'port_dung_sau_6t', 'ti_le_khai_thac_6t', 'tong_port_sau_9t',
           'port_dung_sau_9t', 'ti_le_khai_thac_9t', 'tong_port_sau_12t',
           'port_dung_sau_12t', 'ti_le_khai_thac_12t']]
    df_port_pivot_filter['ky_dau_tu'] =kydautu
    lst_data = df_port_pivot_filter.values.tolist()
    conn = pg.connect("postgresql://%s:%s@%s:%s/%s"% (config['dbs']['dwh_177_public']['user']
                                                      ,config['dbs']['dwh_177_public']['password']
                                                     ,config['dbs']['dwh_177_public']['host']
                                                    ,config['dbs']['dwh_177_public']['port']
                                                    ,config['dbs']['dwh_177_public']['dbname']))
    cur_1 = conn.cursor()
    for i in range(len(lst_data)):
        tupl = (lst_data[i])
        cur_1.execute(
        " INSERT INTO tbl_planning_history(ma_ke_hoach, ky_dau_tu, pop, phuong, quan, chi_nhanh,"
        "vung, dung_luong_trien_khai, perport, tong_port_sau_3t,"
        "port_dung_sau_3t, ti_le_khai_thac_3t, tong_port_sau_6t,"
        "port_dung_sau_6t, ti_le_khai_thac_6t, tong_port_sau_9t,"
        "port_dung_sau_9t, ti_le_khai_thac_9t, tong_port_sau_12t,"
        "port_dung_sau_12t, ti_le_khai_thac_12t) VALUES( %s, %s, %s, %s, %s, %s, %s, %s, %s, %s,"
        "%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
        " ON CONFLICT (ma_ke_hoach, ky_dau_tu, pop, phuong, quan, chi_nhanh,vung)"
        " DO UPDATE SET dung_luong_trien_khai = EXCLUDED.dung_luong_trien_khai, "
        " perport = EXCLUDED.perport, tong_port_sau_3t = EXCLUDED.tong_port_sau_3t,"
        "port_dung_sau_3t = EXCLUDED.port_dung_sau_3t,ti_le_khai_thac_3t = EXCLUDED.ti_le_khai_thac_3t, "
        "tong_port_sau_6t = EXCLUDED.tong_port_sau_6t, port_dung_sau_6t = EXCLUDED.port_dung_sau_6t,"
        "ti_le_khai_thac_6t = EXCLUDED.ti_le_khai_thac_6t,tong_port_sau_9t = EXCLUDED.tong_port_sau_9t, "
        "port_dung_sau_9t = EXCLUDED.port_dung_sau_9t,ti_le_khai_thac_9t = EXCLUDED.ti_le_khai_thac_9t,"
        "tong_port_sau_12t = EXCLUDED.tong_port_sau_12t,port_dung_sau_12t = EXCLUDED.port_dung_sau_12t,"
        "ti_le_khai_thac_12t = EXCLUDED.ti_le_khai_thac_12t;", (tuple(tupl))
        )
    try:
        conn.commit()
        conn.close()
        print("Successfully!!!!")
    except:
        print("Don't save DB")